from __future__ import annotations

import json
import typer

from .core.engine import apply_v1_bundle, doctor_v1, uninstall_v1
from .core.paths import resolve_target

app = typer.Typer(add_completion=False)


@app.command()
def sync(
    target: str = ".",
    force: bool = typer.Option(False, "--force"),
    json_out: bool = typer.Option(False, "--json"),
) -> None:
    target_root = resolve_target(target)
    result = apply_v1_bundle(target_root, dry_run=False, force=force)

    if json_out:
        typer.echo(
            json.dumps(
                {
                    "target": str(result.target),
                    "created": result.created,
                    "overwritten": result.overwritten,
                    "skipped": result.skipped,
                    "conflicts": list(result.conflicts),
                    "manifest": str(result.manifest_path),
                },
                indent=2,
                sort_keys=True,
            )
        )
        raise typer.Exit(code=0 if not result.conflicts else 2)

    typer.echo(f"[vibe-wf] target: {result.target}")
    if result.conflicts:
        typer.echo("[vibe-wf] conflicts detected (use --force to overwrite):")
        for p in result.conflicts:
            typer.echo(f"- {p}")
        raise typer.Exit(code=2)

    typer.echo(
        f"[vibe-wf] done: created={result.created} overwritten={result.overwritten} skipped={result.skipped}"
    )


@app.command()
def init(
    target: str = ".",
    dry_run: bool = typer.Option(False, "--dry-run"),
    force: bool = typer.Option(False, "--force"),
    json_out: bool = typer.Option(False, "--json"),
) -> None:
    target_root = resolve_target(target)
    result = apply_v1_bundle(target_root, dry_run=dry_run, force=force)

    if json_out:
        typer.echo(
            json.dumps(
                {
                    "target": str(result.target),
                    "created": result.created,
                    "overwritten": result.overwritten,
                    "skipped": result.skipped,
                    "conflicts": list(result.conflicts),
                    "manifest": str(result.manifest_path),
                },
                indent=2,
                sort_keys=True,
            )
        )
        raise typer.Exit(code=0 if not result.conflicts else 2)

    typer.echo(f"[vibe-wf] target: {result.target}")
    if result.conflicts:
        typer.echo("[vibe-wf] conflicts detected (use --force to overwrite):")
        for p in result.conflicts:
            typer.echo(f"- {p}")
        raise typer.Exit(code=2)

    typer.echo(
        f"[vibe-wf] done: created={result.created} overwritten={result.overwritten} skipped={result.skipped}"
    )


@app.command()
def doctor(
    target: str = ".",
    json_out: bool = typer.Option(False, "--json"),
) -> None:
    target_root = resolve_target(target)
    result = doctor_v1(target_root)

    if json_out:
        typer.echo(
            json.dumps(
                {
                    "installed": result.installed,
                    "total": result.total,
                    "ok": result.ok,
                    "modified": result.modified,
                    "missing": result.missing,
                },
                indent=2,
                sort_keys=True,
            )
        )
        raise typer.Exit(code=0 if result.installed and result.missing == 0 else 2)

    if not result.installed:
        typer.echo("[vibe-wf] not installed (no manifest)")
        raise typer.Exit(code=2)

    typer.echo(
        f"[vibe-wf] ok={result.ok}/{result.total} modified={result.modified} missing={result.missing}"
    )
    raise typer.Exit(code=0 if result.missing == 0 else 2)


@app.command()
def uninstall(
    target: str = ".",
    dry_run: bool = typer.Option(False, "--dry-run"),
    force: bool = typer.Option(False, "--force"),
    json_out: bool = typer.Option(False, "--json"),
) -> None:
    target_root = resolve_target(target)
    result = uninstall_v1(target_root, dry_run=dry_run, force=force)

    if json_out:
        typer.echo(
            json.dumps(
                {
                    "removed": result.removed,
                    "kept_modified": result.kept_modified,
                    "missing": result.missing,
                },
                indent=2,
                sort_keys=True,
            )
        )
        raise typer.Exit(code=0)

    typer.echo(
        f"[vibe-wf] removed={result.removed} kept_modified={result.kept_modified} missing={result.missing}"
    )


def main() -> None:
    import sys

    if len(sys.argv) == 1:
        from .menu import run_menu

        run_menu()
        return

    app()



from __future__ import annotations

from pathlib import Path


def resolve_target(target: str) -> Path:
    p = Path(target).expanduser()
    # Preserve relative targets (e.g. ".") but normalize for checks.
    return p.resolve()


def ensure_within_target(target_root: Path, path: Path) -> None:
    # Use absolute/normalized paths for boundary checks.
    target_root = target_root.resolve()
    path = path.resolve()

    try:
        path.relative_to(target_root)
    except ValueError as e:
        raise ValueError(f"Refusing to write outside target: {path}") from e



from __future__ import annotations

import json
from dataclasses import dataclass
from importlib.resources import as_file, files
from pathlib import Path


@dataclass(frozen=True)
class BundleSpec:
    schema_version: int
    bundle_id: str
    bundle_version: str
    description: str
    files_root: str
    managed_dir: str
    copy: tuple[str, ...]


def load_v1_bundle() -> tuple[BundleSpec, Path]:

    bundle_pkg = "vibe_wf.bundles.v1"
    manifest_res = files(bundle_pkg).joinpath("manifest.json")

    with as_file(manifest_res) as manifest_path:
        bundle_root = manifest_path.parent
        spec_data = json.loads(manifest_path.read_text(encoding="utf-8"))

    install = spec_data["install"]
    spec = BundleSpec(
        schema_version=int(spec_data["schema_version"]),
        bundle_id=str(spec_data["bundle_id"]),
        bundle_version=str(spec_data["bundle_version"]),
        description=str(spec_data.get("description", "")),
        files_root=str(spec_data.get("files_root", "files")),
        managed_dir=str(spec_data.get("managed_dir", ".vibe-wf")),
        copy=tuple(install.get("copy", [])),
    )

    return spec, bundle_root



from __future__ import annotations

import json
from dataclasses import dataclass
from datetime import datetime, timezone
from pathlib import Path

from .bundle import load_v1_bundle
from .fs import atomic_write_text, atomic_write_bytes, copy_to_backup
from .hashing import sha256_file
from .manifest import ManagedFile, ProjectManifest
from .paths import ensure_within_target


@dataclass(frozen=True)
class ApplyResult:
    created: int
    overwritten: int
    skipped: int
    conflicts: tuple[str, ...]
    target: Path
    manifest_path: Path


@dataclass(frozen=True)
class DoctorResult:
    installed: bool
    total: int
    ok: int
    modified: int
    missing: int


@dataclass(frozen=True)
class UninstallResult:
    removed: int
    kept_modified: int
    missing: int


def apply_v1_bundle(target_root: Path, *, dry_run: bool, force: bool) -> ApplyResult:
    spec, bundle_root = load_v1_bundle()

    files_root = bundle_root / spec.files_root
    state_dir = target_root / spec.managed_dir
    manifest_path = state_dir / "manifest.json"
    version_path = state_dir / "version.json"

    existing: ProjectManifest | None = None
    if manifest_path.exists():
        existing = ProjectManifest.load(manifest_path)

    existing_map = {f.path: f.sha256 for f in existing.managed_files} if existing else {}

    planned: list[tuple[str, str]] = []
    conflicts: list[str] = []

    for rel in spec.copy:
        src = files_root / rel
        if not src.is_file():
            raise FileNotFoundError(f"Bundle is missing file: {rel}")

        dest = target_root / rel
        ensure_within_target(target_root, dest)

        if dest.exists():
            if dest.is_dir():
                conflicts.append(rel)
                planned.append((rel, "conflict"))
                continue

            if sha256_file(dest) == sha256_file(src):
                planned.append((rel, "skip"))
                continue

            if existing_map.get(rel) == sha256_file(dest):
                planned.append((rel, "overwrite"))
                continue

            if force:
                planned.append((rel, "overwrite"))
            else:
                planned.append((rel, "conflict"))
                conflicts.append(rel)
        else:
            planned.append((rel, "create"))

    if conflicts and not force:
        return ApplyResult(
            created=0,
            overwritten=0,
            skipped=0,
            conflicts=tuple(conflicts),
            target=target_root,
            manifest_path=manifest_path,
        )

    created = overwritten = skipped = 0

    if dry_run:
        for _, action in planned:
            if action == "create":
                created += 1
            elif action == "overwrite":
                overwritten += 1
            elif action == "skip":
                skipped += 1
        return ApplyResult(
            created=created,
            overwritten=overwritten,
            skipped=skipped,
            conflicts=tuple(conflicts),
            target=target_root,
            manifest_path=manifest_path,
        )

    state_dir.mkdir(parents=True, exist_ok=True)

    backup_dir: Path | None = None
    if any(action == "overwrite" for _, action in planned):
        ts = datetime.now(timezone.utc).strftime("%Y%m%dT%H%M%SZ")
        backup_dir = state_dir / "backup" / ts
        backup_dir.mkdir(parents=True, exist_ok=True)

    managed_files: list[ManagedFile] = []

    for rel, action in planned:
        src = files_root / rel
        dest = target_root / rel

        if action == "skip":
            skipped += 1
        else:
            if dest.exists() and backup_dir is not None and action == "overwrite":
                copy_to_backup(dest, backup_dir, rel)

            atomic_write_bytes(dest, src.read_bytes())

            if action == "create":
                created += 1
            elif action == "overwrite":
                overwritten += 1

        managed_files.append(ManagedFile(path=rel, sha256=sha256_file(dest)))

    new_manifest = ProjectManifest(
        schema_version=1,
        bundle_id=spec.bundle_id,
        bundle_version=spec.bundle_version,
        managed_files=tuple(managed_files),
    )

    atomic_write_text(manifest_path, json.dumps(
        {
            "schema_version": new_manifest.schema_version,
            "bundle_id": new_manifest.bundle_id,
            "bundle_version": new_manifest.bundle_version,
            "managed_files": [
                {"path": f.path, "sha256": f.sha256} for f in new_manifest.managed_files
            ],
        },
        indent=2,
        sort_keys=True,
    ) + "\n")

    atomic_write_text(
        version_path,
        json.dumps(
            {
                "bundle_id": spec.bundle_id,
                "bundle_version": spec.bundle_version,
                "installed_at": datetime.now(timezone.utc).isoformat(),
            },
            indent=2,
            sort_keys=True,
        )
        + "\n",
    )

    return ApplyResult(
        created=created,
        overwritten=overwritten,
        skipped=skipped,
        conflicts=tuple(conflicts),
        target=target_root,
        manifest_path=manifest_path,
    )


def doctor_v1(target_root: Path) -> DoctorResult:
    spec, _ = load_v1_bundle()

    state_dir = target_root / spec.managed_dir
    manifest_path = state_dir / "manifest.json"

    if not manifest_path.exists():
        return DoctorResult(installed=False, total=0, ok=0, modified=0, missing=0)

    manifest = ProjectManifest.load(manifest_path)

    total = len(manifest.managed_files)
    ok = modified = missing = 0

    for item in manifest.managed_files:
        p = target_root / item.path
        if not p.exists():
            missing += 1
            continue
        if p.is_dir():
            missing += 1
            continue
        if sha256_file(p) != item.sha256:
            modified += 1
            continue
        ok += 1

    return DoctorResult(installed=True, total=total, ok=ok, modified=modified, missing=missing)


def uninstall_v1(target_root: Path, *, dry_run: bool, force: bool) -> UninstallResult:
    spec, _ = load_v1_bundle()

    state_dir = target_root / spec.managed_dir
    manifest_path = state_dir / "manifest.json"
    version_path = state_dir / "version.json"

    if not manifest_path.exists():
        return UninstallResult(removed=0, kept_modified=0, missing=0)

    manifest = ProjectManifest.load(manifest_path)

    removed = kept_modified = missing = 0

    for item in manifest.managed_files:
        p = target_root / item.path
        ensure_within_target(target_root, p)

        if not p.exists():
            missing += 1
            continue
        if p.is_dir():
            missing += 1
            continue

        if sha256_file(p) != item.sha256 and not force:
            kept_modified += 1
            continue

        if not dry_run:
            p.unlink()
        removed += 1

    if not dry_run:
        try:
            manifest_path.unlink()
        except FileNotFoundError:
            pass
        try:
            version_path.unlink()
        except FileNotFoundError:
            pass

    return UninstallResult(removed=removed, kept_modified=kept_modified, missing=missing)



from __future__ import annotations

import json
from dataclasses import dataclass
from pathlib import Path


@dataclass(frozen=True)
class ManagedFile:
    path: str
    sha256: str


@dataclass(frozen=True)
class ProjectManifest:
    schema_version: int
    bundle_id: str
    bundle_version: str
    managed_files: tuple[ManagedFile, ...]

    @staticmethod
    def load(path: Path) -> "ProjectManifest":
        data = json.loads(path.read_text(encoding="utf-8"))
        files = tuple(ManagedFile(**item) for item in data.get("managed_files", []))
        return ProjectManifest(
            schema_version=int(data["schema_version"]),
            bundle_id=str(data["bundle_id"]),
            bundle_version=str(data["bundle_version"]),
            managed_files=files,
        )

    def dump(self, path: Path) -> None:
        data = {
            "schema_version": self.schema_version,
            "bundle_id": self.bundle_id,
            "bundle_version": self.bundle_version,
            "managed_files": [
                {"path": f.path, "sha256": f.sha256} for f in self.managed_files
            ],
        }
        path.write_text(json.dumps(data, indent=2, sort_keys=True) + "\n", encoding="utf-8")



from __future__ import annotations

import json

from rich.console import Console
from rich.panel import Panel
from rich.prompt import Prompt

from .core.engine import apply_v1_bundle, doctor_v1, uninstall_v1
from .core.paths import resolve_target


def _print_banner(console: Console) -> None:
    console.print(
        Panel.fit(
            "VIBE-WF\n\n项目级工作流引导工具（OpenCode + oh-my-opencode）\n\n承诺：只修改项目内文件，不修改系统级配置",
            border_style="cyan",
        )
    )


def run_menu() -> None:
    console = Console()
    _print_banner(console)

    target = "."

    while True:
        console.print("\n请选择功能：")
        console.print("  1. 完整初始化（写入项目内资产与文档）")
        console.print("  2. 同步更新（只覆盖已托管文件；遇到用户改动默认提示冲突）")
        console.print("  +. Doctor（只读诊断）")
        console.print("  -. 卸载（移除工具托管文件）")
        console.print("  T. 更改目标目录（当前：%s）" % target)
        console.print("  Q. 退出")

        choice = Prompt.ask("?", default="Q").strip().lower()

        if choice == "q":
            return

        if choice == "t":
            new_target = Prompt.ask("目标目录", default=target).strip()
            if new_target:
                target = new_target
            continue

        target_root = resolve_target(target)

        if choice == "1":
            res = apply_v1_bundle(target_root, dry_run=False, force=False)
            if res.conflicts:
                console.print("检测到冲突（默认不覆盖）：", style="yellow")
                for p in res.conflicts:
                    console.print(f"- {p}")
                force = Prompt.ask("是否覆盖冲突文件？(y/N)", default="N").strip().lower()
                if force in {"y", "yes"}:
                    res = apply_v1_bundle(target_root, dry_run=False, force=True)
            console.print(
                json.dumps(
                    {
                        "target": str(res.target),
                        "created": res.created,
                        "overwritten": res.overwritten,
                        "skipped": res.skipped,
                        "conflicts": list(res.conflicts),
                    },
                    indent=2,
                    sort_keys=True,
                )
            )
            continue

        if choice == "2":
            res = apply_v1_bundle(target_root, dry_run=False, force=False)
            if res.conflicts:
                console.print("检测到冲突（说明用户修改过或存在同名文件）：", style="yellow")
                for p in res.conflicts:
                    console.print(f"- {p}")
                force = Prompt.ask("是否强制覆盖这些冲突文件？(y/N)", default="N").strip().lower()
                if force in {"y", "yes"}:
                    res = apply_v1_bundle(target_root, dry_run=False, force=True)
            console.print(
                json.dumps(
                    {
                        "target": str(res.target),
                        "created": res.created,
                        "overwritten": res.overwritten,
                        "skipped": res.skipped,
                        "conflicts": list(res.conflicts),
                    },
                    indent=2,
                    sort_keys=True,
                )
            )
            continue

        if choice == "+":
            doc = doctor_v1(target_root)
            console.print(
                json.dumps(
                    {
                        "installed": doc.installed,
                        "total": doc.total,
                        "ok": doc.ok,
                        "modified": doc.modified,
                        "missing": doc.missing,
                    },
                    indent=2,
                    sort_keys=True,
                )
            )
            continue

        if choice == "-":
            res = uninstall_v1(target_root, dry_run=False, force=False)
            if res.kept_modified:
                console.print("发现被用户修改过的托管文件（默认保留）：", style="yellow")
                force = Prompt.ask("是否强制删除这些已修改文件？(y/N)", default="N").strip().lower()
                if force in {"y", "yes"}:
                    res = uninstall_v1(target_root, dry_run=False, force=True)
            console.print(
                json.dumps(
                    {
                        "removed": res.removed,
                        "kept_modified": res.kept_modified,
                        "missing": res.missing,
                    },
                    indent=2,
                    sort_keys=True,
                )
            )
            continue

        console.print("未知选项。", style="red")



from __future__ import annotations

import hashlib
from pathlib import Path


def sha256_bytes(data: bytes) -> str:
    h = hashlib.sha256()
    h.update(data)
    return h.hexdigest()


def sha256_file(path: Path) -> str:
    h = hashlib.sha256()
    with path.open("rb") as f:
        for chunk in iter(lambda: f.read(1024 * 1024), b""):
            h.update(chunk)
    return h.hexdigest()



from __future__ import annotations

from pathlib import Path

from vibe_wf.core.engine import apply_v1_bundle, doctor_v1, uninstall_v1


def test_init_dry_run_reports_conflicts_for_existing_files(tmp_path: Path) -> None:
    (tmp_path / "AGENTS.md").write_text("user", encoding="utf-8")

    res = apply_v1_bundle(tmp_path, dry_run=True, force=False)
    assert res.conflicts
    assert "AGENTS.md" in res.conflicts


def test_apply_then_doctor_then_uninstall_roundtrip(tmp_path: Path) -> None:
    res = apply_v1_bundle(tmp_path, dry_run=False, force=False)
    assert not res.conflicts

    manifest_path = tmp_path / ".vibe-wf" / "manifest.json"
    assert manifest_path.exists()

    doc = doctor_v1(tmp_path)
    assert doc.installed is True
    assert doc.missing == 0

    uninstall = uninstall_v1(tmp_path, dry_run=False, force=False)
    assert uninstall.kept_modified == 0

    assert not manifest_path.exists()


def test_uninstall_keeps_modified_files_without_force(tmp_path: Path) -> None:
    res = apply_v1_bundle(tmp_path, dry_run=False, force=False)
    assert not res.conflicts

    target = tmp_path / "AGENTS.md"
    target.write_text("modified", encoding="utf-8")

    uninstall = uninstall_v1(tmp_path, dry_run=False, force=False)
    assert uninstall.kept_modified >= 1
    assert target.exists()


def test_engine_never_writes_outside_target(tmp_path: Path) -> None:
    outside = tmp_path.parent / "outside"
    outside.mkdir()

    marker = outside / "marker.txt"
    marker.write_text("pre", encoding="utf-8")

    apply_v1_bundle(tmp_path, dry_run=False, force=False)

    assert marker.read_text(encoding="utf-8") == "pre"



from __future__ import annotations

import os
import shutil
import tempfile
from pathlib import Path


def atomic_write_bytes(path: Path, data: bytes) -> None:
    path.parent.mkdir(parents=True, exist_ok=True)

    fd, tmp = tempfile.mkstemp(prefix=path.name + ".", suffix=".tmp", dir=str(path.parent))
    try:
        with os.fdopen(fd, "wb") as f:
            f.write(data)
        os.replace(tmp, path)
    finally:
        try:
            os.unlink(tmp)
        except FileNotFoundError:
            pass


def atomic_write_text(path: Path, text: str, encoding: str = "utf-8") -> None:
    atomic_write_bytes(path, text.encode(encoding))


def copy_to_backup(src: Path, backup_root: Path, rel_path: str) -> Path:
    dest = backup_root / rel_path
    dest.parent.mkdir(parents=True, exist_ok=True)
    shutil.copy2(src, dest)
    return dest
